#!/usr/local/bin/perl
# mod_tri_wrong.pl
# This perl script reads a sorted (!) list of enable-nets with their 
# associated tristate buffers. 
# (List is produced by dc_shell-script).
# Example: 
#
# n123 n732 databus_tri[2]
# n123 n732 databus_tri[3]
# n678 n789 addrbus_tri[3]
#
# Out of this list it generates a dc_shell script that inserts inverters
# into the netlist, connects the input of the inverters to the enable 
# net (e.g. n678) and the output to a not-enable net (e.g. n678_n) that is 
# connected to the opposite gate enable of the tristate buffer.
#
# Required DC_SHELL variables:
#  LIB_INV_CELL  - Inverter cell to be included
#  INV_IN_PIN
#  INV_OUT_PIN
#  EN2_PIN       - Pin to be reconnected

$act_net= ""; # No active net!

while (<>)
{
  if (/(\S+)\s+(\S+)\s+(\S+)/) # Line contains valid entry
  { 
    $net= $1;
    $wrong_net= $2;
    $cell= $3;
    if ($net ne $act_net) 
    {
      # New net: insert new inverter...
      print("create_cell " . $net . "_inv  LIB_INV_CELL\n");
      print("create_net  " . $net . "_n\n");
      print("connect_net " . $net . "   " . $net . "_inv/ + INV_IN_PIN\n");
      print("connect_net " . $net . "_n " . $net . "_inv/ + INV_OUT_PIN\n\n");
      $act_net= $net; # update activ net
    }
    # Disconnect wrong enable net
    print("disconnect_net " . $wrong_net . " " . $cell . "/ + EN2_PIN\n");
    # connect new enable net to 
    print("connect_net " . $net . "_n " . $cell . "/ + EN2_PIN\n");
  }
}
